/*
 * neopixel web gui stop light, D5 as signal for three (3) ws2812s
 * stop light changes states via commands from a webpage gui
 * 
 * Code adapted from:
 * Rui Santos 
 * Complete project details at https://randomnerdtutorials.com  
 * 
 */

// Load Wi-Fi library
#include <ESP8266WiFi.h>
#include <Adafruit_NeoPixel.h>

// Replace with your network credentials
const char* ssid     = "your_sside";
const char* password = "your_passwd";

#define PIN D5 // neopixel

// Parameter 1 = number of pixels in strip
// Parameter 2 = pin number (most are valid)
// Parameter 3 = pixel type flags, add together as needed:
//   NEO_KHZ800  800 KHz bitstream (most NeoPixel products w/WS2812 LEDs)
//   NEO_KHZ400  400 KHz (classic 'v1' (not v2) FLORA pixels, WS2811 drivers)
//   NEO_GRB     Pixels are wired for GRB bitstream (most NeoPixel products)
//   NEO_RGB     Pixels are wired for RGB bitstream (v1 FLORA pixels, not v2)
Adafruit_NeoPixel strip = Adafruit_NeoPixel(3, PIN, NEO_RGB + NEO_KHZ800);

// define the color codes by name to make it easier to call later on
uint32_t green = strip.Color(255, 0, 0);
uint32_t red = strip.Color(0, 255, 0);
uint32_t yellow = strip.Color(255, 100, 0);

// Set web server port number to 80
WiFiServer server(80);

// Variable to store the HTTP request
String header;

// Auxiliar variables to store the current output state
String redState = "off";
String yellowState = "off";
String greenState = "off";

// Current time
unsigned long currentTime = millis();
// Previous time
unsigned long previousTime = 0; 
// Define timeout time in milliseconds (example: 2000ms = 2s)
const long timeoutTime = 2000;

void setup() {
  Serial.begin(115200);

  strip.begin();
  strip.show();   // initialize all pixels to 'off'

  // Connect to Wi-Fi network with SSID and password
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  // Print local IP address and start web server
  Serial.println("");
  Serial.println("WiFi connected.");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
  server.begin();
}

void loop(){
  WiFiClient client = server.available();   // Listen for incoming clients

  if (client) {                             // If a new client connects,
    Serial.println("New Client.");          // print a message out in the serial port
    String currentLine = "";                // make a String to hold incoming data from the client
    currentTime = millis();
    previousTime = currentTime;
    while (client.connected() && currentTime - previousTime <= timeoutTime) { // loop while the client's connected
      currentTime = millis();         
      if (client.available()) {             // if there's bytes to read from the client,
        char c = client.read();             // read a byte, then
        Serial.write(c);                    // print it out the serial monitor
        header += c;
        if (c == '\n') {                    // if the byte is a newline character
          // if the current line is blank, you got two newline characters in a row.
          // that's the end of the client HTTP request, so send a response:
          if (currentLine.length() == 0) {
            // HTTP headers always start with a response code (e.g. HTTP/1.1 200 OK)
            // and a content-type so the client knows what's coming, then a blank line:
            client.println("HTTP/1.1 200 OK");
            client.println("Content-type:text/html");
            client.println("Connection: close");
            client.println();
            
            // turns the neopixels on and off
            if (header.indexOf("GET /red/on") >= 0) {
              Serial.println("Red on");
              redState = "on";
              yellowState = "off";
              greenState = "off";
              strip.clear();
              strip.setPixelColor(0, red);
              strip.show();
            } else if (header.indexOf("GET /red/off") >= 0) {
              Serial.println("Red off");
              redState = "off";
              yellowState = "off";
              greenState = "off";
              strip.clear();
              strip.show();
            } else if (header.indexOf("GET /yellow/on") >= 0) {
              Serial.println("Yellow on");
              yellowState = "on";
              redState = "off";
              greenState = "off";
              strip.clear();
              strip.setPixelColor(1, yellow);
              strip.show();
            } else if (header.indexOf("GET /yellow/off") >= 0) {
              Serial.println("Yellow off");
              yellowState = "off";
              redState = "off";
              greenState = "off";
              strip.clear();
              strip.show();
            } else if (header.indexOf("GET /green/on") >= 0) {
              Serial.println("Green on");
              greenState = "on";
              redState = "off";
              yellowState = "off";
              strip.clear();
              strip.setPixelColor(2, green);
              strip.show();
            } else if (header.indexOf("GET /green/off") >= 0) {
              Serial.println("Green off");
              greenState = "off";
              redState = "off";
              yellowState = "off";
              strip.clear();
              strip.show();
            }

            // Display the HTML web page
            client.println("<!DOCTYPE html><html>");
            client.println("<head><meta name=\"viewport\" content=\"width=device-width, initial-scale=1\">");
            client.println("<link rel=\"icon\" href=\"data:,\">");
            // CSS to style the on/off buttons 
            // Feel free to change the background-color and font-size attributes to fit your preferences
            client.println("<style>html { font-family: Helvetica; display: inline-block; margin: 0px auto; text-align: center;}");
            client.println(".button { background-color: #195B6A; border: none; color: white; padding: 16px 40px;");
            client.println("text-decoration: none; font-size: 30px; margin: 2px; cursor: pointer;}");
            client.println(".button2 {background-color: #77878A;}</style></head>");
            
            // Web Page Heading
            client.println("<body><h1>Stoplight Server</h1>");
            
            // Display current state, and ON/OFF buttons for Red  
            client.println("<p>Red: " + redState + "</p>");
            // If the redState is off, it displays the ON button       
            if (redState=="off") {
              client.println("<p><a href=\"/red/on\"><button class=\"button\">ON</button></a></p>");
            } else {
              client.println("<p><a href=\"/red/off\"><button class=\"button button2\">OFF</button></a></p>");
            } 
               
            // Display current state, and ON/OFF buttons for Yellow 
            client.println("<p>Yellow: " + yellowState + "</p>");
            // If the yellowState is off, it displays the ON button       
            if (yellowState=="off") {
              client.println("<p><a href=\"/yellow/on\"><button class=\"button\">ON</button></a></p>");
            } else {
              client.println("<p><a href=\"/yellow/off\"><button class=\"button button2\">OFF</button></a></p>");
            }

            // Display current state, and ON/OFF buttons for Green 
            client.println("<p>Green: " + greenState + "</p>");
            // If the greenState is off, it displays the ON button       
            if (greenState=="off") {
              client.println("<p><a href=\"/green/on\"><button class=\"button\">ON</button></a></p>");
            } else {
              client.println("<p><a href=\"/green/off\"><button class=\"button button2\">OFF</button></a></p>");
            }

            client.println("</body></html>");
            
            // The HTTP response ends with another blank line
            client.println();
            // Break out of the while loop
            break;
          } else { // if you got a newline, then clear currentLine
            currentLine = "";
          }
        } else if (c != '\r') {  // if you got anything else but a carriage return character,
          currentLine += c;      // add it to the end of the currentLine
        }
      }
    }
    // Clear the header variable
    header = "";
    // Close the connection
    client.stop();
    Serial.println("Client disconnected.");
    Serial.println("");
  }
}
